/*
 * Blackmore's Channel Logger - A channel logger module for Inspircd
 * Copyright (C) 2012 Simeon Simeonov
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "inspircd.h"


class ModuleBChanLog : public Module
{


private:

  std::multimap<std::string, std::string> channels;


public:

  ModuleBChanLog()
  {
    
    Implementation eventlist[] = { I_OnRehash, I_OnUserMessage, I_OnUserNotice };
    ServerInstance->Modules->Attach(eventlist, this, 3);
    
    OnRehash(NULL);

  }

  
  virtual ~ModuleBChanLog()
  {

  }

  
  virtual void 
  OnRehash(User *user)
  {

    ConfigReader conf;
    
    channels.clear();

    for (int i = 0; i < conf.Enumerate("bchanlog"); i++) {
 
      std::string channel;
      std::string type;
      
      channel = conf.ReadValue("bchanlog", "channel", i);
      type = conf.ReadValue("bchanlog", "type", i);
      
      if (channel.empty() || type.empty()) {
        ServerInstance->Logs->Log("m_bchanlog", 
			    DEFAULT, 
			    "Malformed bchanlog tag, ignoring");
        continue;
      }
      
      if (channels.find(channel) != channels.end()) {
        ServerInstance->Logs->Log("m_bchanlog", 
			    DEFAULT, 
			    "%s already added. Skipping...", channel.c_str());
        continue;
      }

      channels.insert(std::make_pair(channel, type));
      ServerInstance->Logs->Log("m_bchanlog", DEFAULT, 
			  "Logging %s to: %s", 
			  channel.c_str(), 
			  type.c_str());
      
    }
    
  }

  
  virtual void 
  OnUserMessage (User *user, 
	       void *dest, 
	       int target_type, 
	       const std::string &text, 
	       char status, 
	       const CUList &exempt_list) {

    if (target_type != TYPE_CHANNEL) {
      return;
    }

    Channel *chan = (Channel *) dest;

    if (channels.find(chan->name) != channels.end()) {
      
      ServerInstance->Logs->Log(channels.find(chan->name)->second, 
			  DEFAULT, 
			  "%s - %s -> %s",
			  chan->name.c_str(),
			  user->nick.c_str(),
			  text.c_str());
    }

  }


  virtual void 
  OnUserNotice (User *user, 
	      void *dest, 
	      int target_type, 
	      const std::string &text, 
	      char status, 
	      const CUList &exempt_list) {

    if (target_type != TYPE_CHANNEL) {
      return;
    }
    
    Channel *chan = (Channel *) dest;
    
    if (channels.find(chan->name) != channels.end()) {
      ServerInstance->Logs->Log(channels.find(chan->name)->second, 
			  DEFAULT, 
			  "%s - %s <> %s",
			  chan->name.c_str(),
			  user->nick.c_str(),
			  text.c_str());
    }
    
    
  }


  virtual Version 
  GetVersion()
  {
    return Version("Logs channel messages on different channels.", VF_VENDOR);
  }
  
  
};

MODULE_INIT(ModuleBChanLog)
